/** @file
 * @brief TCPService class header. This class represents a TCP client.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#ifndef TCPSERVICE_H
#define TCPSERVICE_H

#include "logger.h"
#include "ip_service.h"

#include <boost/shared_ptr.hpp>
#include <boost/asio.hpp>
#include <boost/asio/ssl.hpp>

typedef boost::asio::ssl::stream<boost::asio::ip::tcp::socket> SSLStream;

class TCPService : public IPService
{

private:

    boost::shared_ptr<SSLStream> Socket;
    // IOService has to be a member, otherwise the socket.close() operation will throw "mutex: Invalid argument"
    // Bug in boost::asio ? Cause no docs found concerning this issue. The socket.close() operation is trying to clean up io_service.
    boost::asio::io_service IOService;

public:

    //typedef boost::shared_ptr<TCPService> Ptr;

    TCPService();

    ~TCPService();

    virtual void connect(const std::string& _hostname, const std::string& _port) throw (boost::system::system_error);

    std::string read_from_socket() throw (boost::system::system_error);

    int write_to_socket(const std::string& data) throw (boost::system::system_error);

    int close() throw (boost::system::system_error);
};

#endif
